# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 TU Wien.
#
# Invenio-Requests is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""JSONSchema directory."""

# TODO it'd be nice to re-use the #agent $ref, but that's defined in
#      rdm-records
#      -> move that $ref to a more generic package? records-resources maybe?
